/* monitor.cc
 * Copyright (C) 2001-2 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gnome.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "filesel.h"
#include "monitor.h"
#include "discover.h"
#include "diagram.h"
#include "preferences.h"
#include "scheduler.h"
#include "sessionlist.h"
#include "message.h"
#include "features.h"
#include "main.h"
#include "misc.h"
#include "gui.h"
#include "data.h"
#include "print.h"
#include "stats.h"
#include "device.h"
#include "devicesidebar.h"
#include "searchsidebar.h"
#include "viewsidebar.h"
#include "filtersidebar.h"
#include "session.h"
#include "subnets.h"
#include "tasklist.h"
#include "eventlog.h"

#include "licon.xpm"
#include "ssummary.xpm"
#include "srouter.xpm"
#include "sbrouter.xpm"
#include "ssearch.xpm"
#include "sview.xpm"
#include "scomputer.xpm"
#include "scloud.xpm"
#include "sswitch.xpm"
#include "ssubnet.xpm"
#include "sfitall.xpm"
#include "sfitsel.xpm"

//static GnomeUIInfo settings1_menu_uiinfo[];

const gchar *authors[] = { "Paul Coates <Paul.Coates@ncl.ac.uk>", NULL };

static gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	((Monitor*)data)->main->quit();
	return (TRUE);
}

// File Callbacks

static void new_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(1); }
static void open_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(2); }
static void import_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->filesel->show(); }
static void import_ok_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->import_action(); }
static void import_cancel_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->filesel->hide(); }

static void import_list_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->filesel2->show(); }
static void import_list_ok_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->import_list_action(); }
static void import_list_cancel_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->filesel2->hide(); }
static void save_cb(GtkWidget *w, gpointer data)
{
	((Monitor*)data)->main->session->action = 0;
	((Monitor*)data)->main->session->have_title();
}
static void saveas_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*) data;
	m->main->session->action = 3;
	strcpy(m->main->session->orig_session_name, m->main->session->session_name);
	m->main->session->save_session_list.init(m->main->session->datadir);
	m->main->session->save_session_list.show();
}
static void print_cb(GtkWidget *w, gpointer data)
{
//	gnome_triggers_do("Nomad", "pingsnd",  NULL);
}
static void exit_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->session->new_session(4); }


// View Callbacks

static void center_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->viewer->fitToWindow(); }
static void selected_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->viewer->fitToSelected(); }
static void select_all_cb(GtkWidget *w, gpointer data)
{
	Monitor *m = (Monitor*)data;
	m->select_all(TRUE);
	m->main->session->data->select(TRUE);
	m->viewer->draw();
}
static void bunchall_cb(GtkWidget *w, gpointer data)
{ Monitor *m = (Monitor*)data; m->main->session->data->group_all(device_group_bunch); m->main->gui->update_views(); }
static void starall_cb(GtkWidget *w, gpointer data)
{ Monitor *m = (Monitor*)data; m->main->session->data->group_all(device_group_star); m->main->gui->update_views(); }
static void show_all_cb(GtkWidget *w, gpointer data)
{ Monitor *m = (Monitor*)data; m->main->session->data->show_all(); m->main->gui->update_views(); }
static void show_all_leaf_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->viewer->all_leaf_networks(TRUE); }
static void hide_all_leaf_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->viewer->all_leaf_networks(FALSE); }
static void set_default_network_types_cb(GtkWidget *w, gpointer data)
{	((Monitor*)data)->main->session->data->network_type_auto_all();
	((Monitor*)data)->main->gui->update_views();
}

// Settings Callbacks

static void sidepanel_cb(GtkWidget *w, gpointer data)
{	((Monitor*)data)->sidebar_active(); }
static void statusbar_cb(GtkWidget *w, gpointer data)
{	((Monitor*)data)->statusbar_active(); }
static void labels_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->labels_active(); }
static void colour_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->colour_active(); }
static void preferences_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->preferences->show(); }


// Windows Callbacks

static void task_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->tasklist->show(); }
static void subnets_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->subnets->show(); }
static void discover_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->discover->show(); }
static void eventlog_cb(GtkWidget *w, gpointer data)
{ ((Monitor*)data)->main->gui->eventlog->show(); }


// Help Callbacks

static void homepage_cb(GtkWidget *w, gpointer data)
{ gnome_url_show("http://netmon.ncl.ac.uk/"); }
static void manual_cb(GtkWidget *w, gpointer data)
{ // gnome_help_goto(NULL, "ghelp:nomad");
	gnome_url_show("http://netmon.ncl.ac.uk/help/"); }
static void about_cb(GtkWidget *w, gpointer data)
{ 
	static GtkWidget *dialog = NULL;
	GtkWidget *app;

	app = ((Monitor*)data)->mainwindow;

	if (dialog != NULL)
	{
		g_assert(GTK_WIDGET_REALIZED(dialog));
		gdk_window_show(dialog->window);
		gdk_window_raise(dialog->window);
	}else
	{
		gchar *fname = user_file("logo.png");
		dialog = gnome_about_new(_("Nomad"), VERSION,
			_("Copyright (C) 2001-2 Newcastle University"),
			authors, _("Network Mapping & Monitoring Tool"), fname);
		g_free(fname);

		gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
		gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(app));
		gtk_widget_show(dialog);
	}
}

static GnomeUIInfo import1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Device List"),
    NULL,
    (gpointer) import_list_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Device Details"),
    NULL,
    (gpointer) import_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo file1_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_NEW_ITEM (N_("_New"), NULL, new_cb, NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM (open_cb, NULL),
  {
    GNOME_APP_UI_SUBTREE, N_("_Import"),
    NULL,
		import1_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_DATA, sfitsel_xpm,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_MENU_SAVE_ITEM (save_cb, NULL),
  GNOMEUIINFO_MENU_SAVE_AS_ITEM (saveas_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
#ifdef BLEEDING_EDGE
  GNOMEUIINFO_MENU_PRINT_ITEM (print_cb, NULL),
  GNOMEUIINFO_SEPARATOR,
#endif
  GNOMEUIINFO_MENU_EXIT_ITEM (exit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo view1_menu_uiinfo[] =
{
	{
    GNOME_APP_UI_ITEM, N_("_Fit to Window"),
    NULL,
    (gpointer) center_cb, NULL, NULL,
    GNOME_APP_PIXMAP_DATA, sfitall_xpm,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Fit to _Selected"),
    NULL,
    (gpointer) selected_cb, NULL, NULL,
    GNOME_APP_PIXMAP_DATA, sfitsel_xpm,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Select _All"),
    NULL,
    (gpointer) select_all_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("_Bunch All Networks"),
    NULL,
    (gpointer) bunchall_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("S_tar All Networks"),
    NULL,
    (gpointer) starall_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Show All _Devices"),
    NULL,
    (gpointer) show_all_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Show All Leaf Networks"),
    NULL,
    (gpointer) show_all_leaf_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Hide All Leaf Networks"),
    NULL,
    (gpointer) hide_all_leaf_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Apply Default Network Types"),
    NULL,
    (gpointer) set_default_network_types_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo settings1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_TOGGLEITEM, N_("_Side Panel"), NULL,
    (gpointer) sidepanel_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("S_tatus Bar"), NULL,
    (gpointer) statusbar_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("_Labels"), NULL,
    (gpointer) labels_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_TOGGLEITEM, N_("_Coloured Links"), NULL,
    (gpointer) colour_cb, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, NULL,
    0, (GdkModifierType) 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_PREFERENCES_ITEM (preferences_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo windows1_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("_Task List"), NULL,
    (gpointer) task_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Subnets"), NULL,
    (gpointer) subnets_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
    0, (GdkModifierType) 0, NULL
  },
#ifdef BLEEDING_EDGE
  {
    GNOME_APP_UI_ITEM, N_("_Discover"), NULL,
    (gpointer) discover_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
    0, (GdkModifierType) 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("_Event Log"), NULL,
    (gpointer) eventlog_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP,
    0, (GdkModifierType) 0, NULL
  },
#endif
  GNOMEUIINFO_END
};

static GnomeUIInfo help1_menu_uiinfo[] =
{
	{
		GNOME_APP_UI_ITEM, N_("_Homepage"), NULL,
		(gpointer) homepage_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
		0, (GdkModifierType) 0, NULL
	},
	{
		GNOME_APP_UI_ITEM, N_("_Manual"), NULL,
		(gpointer) manual_cb, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BOOK_OPEN,
		0, (GdkModifierType) 0, NULL
	},
	GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo menubar1_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file1_menu_uiinfo),
  GNOMEUIINFO_MENU_VIEW_TREE (view1_menu_uiinfo),
  GNOMEUIINFO_MENU_SETTINGS_TREE (settings1_menu_uiinfo),
  GNOMEUIINFO_MENU_WINDOWS_TREE (windows1_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help1_menu_uiinfo),
  GNOMEUIINFO_END
};

/***************************************************************************
 Monitor Class
 ***************************************************************************/

Monitor::Monitor(Main *m)
{
	GtkWidget *vbox77;
	GtkWidget *separator;
	gchar *str;

	main = m;

	// set default preferences
	side_panel =
		gconf_client_get_bool(main->client, CONFIG_SHOW_SIDEBAR, NULL);
	status_bar =
		gconf_client_get_bool(main->client, CONFIG_SHOW_STATUSBAR, NULL);

  tooltips = gtk_tooltips_new ();

  mainwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (mainwindow), "Nomad");
	gtk_widget_set_usize(mainwindow, 640, 480);
	gtk_widget_realize(mainwindow);
	window_icon(mainwindow, licon_xpm);
	gdk_window_set_icon_name(mainwindow->window, "Nomad");
	gtk_signal_connect(GTK_OBJECT(mainwindow), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), this);
	gtk_signal_connect(GTK_OBJECT(mainwindow), "destroy",
		GTK_SIGNAL_FUNC(delete_event), this);

  pixmap[0] = create_pixmap(mainwindow, ssummary_xpm);
  pixmap[1] = create_pixmap(mainwindow, srouter_xpm);
  pixmap[2] = create_pixmap(mainwindow, ssearch_xpm);
  pixmap[3] = create_pixmap(mainwindow, sview_xpm);
  pixmap[4] = create_pixmap(mainwindow, sswitch_xpm);
  pixmap[5] = create_pixmap(mainwindow, scomputer_xpm);
  pixmap[6] = create_pixmap(mainwindow, scloud_xpm);
  pixmap[7] = create_pixmap(mainwindow, ssubnet_xpm);
  pixmap[8] = create_pixmap(mainwindow, srouter_xpm);
  pixmap[9] = create_pixmap(mainwindow, ssummary_xpm);
  pixmap[10] = create_pixmap(mainwindow, sbrouter_xpm);

	filesel = new FileSel();
	filesel2 = new FileSel();
	viewer = new Viewer(mainwindow, main);
	devicesidebar = new DeviceSideBar(main);
	searchsidebar = new SearchSideBar(main);
	viewsidebar = new ViewSideBar(main);
	filtersidebar = new FilterSideBar(main);

	// these preferences go after the viewer is created
	viewer->diagram->showlabels =
		gconf_client_get_bool(main->client, CONFIG_DIAGRAM_SHOW_LABELS, NULL);
	viewer->diagram->showcolourlines =
		gconf_client_get_bool(main->client, CONFIG_DIAGRAM_SHOW_LINE_COLORS, NULL);

	gtk_container_add (GTK_CONTAINER (devicesidebar->button97), pixmap[9]);

	vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (mainwindow), vbox1);

  menubar = gtk_menu_bar_new();
  gtk_widget_show(menubar);
  gtk_box_pack_start(GTK_BOX(vbox1), menubar, FALSE, FALSE, 0);
  gnome_app_fill_menu_with_data (GTK_MENU_SHELL (menubar), menubar1_uiinfo,
		NULL, FALSE, 0, this);

	hpaned1 = gtk_hpaned_new ();
  gtk_widget_show (hpaned1);
  gtk_box_pack_start (GTK_BOX (vbox1), hpaned1, TRUE, TRUE, 0);
  gtk_paned_set_gutter_size (GTK_PANED (hpaned1), 10);
	gtk_paned_set_handle_size (GTK_PANED (hpaned1), 10);
  gtk_paned_set_position (GTK_PANED (hpaned1), 160);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_paned_pack1 (GTK_PANED (hpaned1), notebook1, FALSE, TRUE);

  gtk_container_add (GTK_CONTAINER (notebook1), devicesidebar->vbox67);

  vbox77 = gtk_vbox_new (FALSE, 4);
  gtk_widget_show (vbox77);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox77);
#ifdef BLEEDING_EDGE
  gtk_container_add (GTK_CONTAINER (notebook1), searchsidebar->vbox12);
#endif

	gtk_box_pack_start (GTK_BOX (vbox77), viewsidebar->vbox67, TRUE, TRUE, 0);
#ifdef BLEEDING_EDGE
  separator = gtk_hseparator_new ();
  gtk_widget_show (separator);
  gtk_box_pack_start (GTK_BOX (vbox77), separator, FALSE, TRUE, 2);
	gtk_box_pack_start (GTK_BOX (vbox77), filtersidebar->vbox56, TRUE, TRUE, 0);
#endif

  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), pixmap[1]);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), pixmap[3]);
#ifdef BLEEDING_EDGE
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
		gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), pixmap[2]);
#endif

	notebook2 = gtk_notebook_new ();
  gtk_widget_show (notebook2);
  gtk_paned_pack2 (GTK_PANED (hpaned1), notebook2, TRUE, FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK (notebook2), FALSE);
	gtk_notebook_set_show_border(GTK_NOTEBOOK (notebook2), FALSE);

  gtk_container_add (GTK_CONTAINER (notebook2), viewer->vbox3);

	create_statusbar(vbox1);

//	separatorbar = gtk_hseparator_new();
//  gtk_widget_show(separatorbar);
//  gtk_box_pack_end(GTK_BOX(vbox1), separatorbar, FALSE, TRUE, 0);

  gtk_object_set_data (GTK_OBJECT (mainwindow), "tooltips", tooltips);

// Apply Preferences

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[0].widget), side_panel);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[1].widget), status_bar);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[2].widget),
		viewer->diagram->showlabels);

	gtk_check_menu_item_set_active(
		GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[3].widget),
		viewer->diagram->showcolourlines);

	filesel->setTitle("Import File");
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel->window)->ok_button), "clicked", GTK_SIGNAL_FUNC(import_ok_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel->window)->cancel_button), "clicked", GTK_SIGNAL_FUNC(import_cancel_cb), this);

	filesel2->setTitle("Import Device List");
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel2->window)->ok_button), "clicked", GTK_SIGNAL_FUNC(import_list_ok_cb), this);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel2->window)->cancel_button), "clicked", GTK_SIGNAL_FUNC(import_list_cancel_cb), this);
}

Monitor::~Monitor()
{
	if (viewer!=NULL) delete viewer;
	if (devicesidebar!=NULL) delete devicesidebar;
	if (searchsidebar!=NULL) delete searchsidebar;
	if (viewsidebar!=NULL) delete viewsidebar;
	if (filesel!=NULL) delete filesel;
	if (filesel2!=NULL) delete filesel;
}

void Monitor::show()
{
	gtk_widget_show(mainwindow);
	gdk_window_raise(mainwindow->window);
}

void Monitor::hide()
{
	gtk_widget_hide(mainwindow);
}

void Monitor::select_all(gboolean val)
{
	Link *ln;
	Device *r;
	gboolean ch;

	for (ln = main->session->data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		ch = r->selection;
		r->selection = FALSE;
		if (r->shown==TRUE) r->selection=val;
		if (ch!=r->selection) select_action(r);
	}
}

void Monitor::select_action(Device *r)
{
	gint row;

	row = gtk_clist_find_row_from_data (GTK_CLIST(devicesidebar->clist31), r);
	if (row==-1) return;

	if (r->selection==TRUE)
		gtk_clist_select_row(GTK_CLIST(devicesidebar->clist31), row, 0);
	else
		gtk_clist_unselect_row(GTK_CLIST(devicesidebar->clist31), row, 0);
}

void Monitor::sidebar_active()
{
	gconf_client_set_bool(main->client, CONFIG_SHOW_SIDEBAR,
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[0].widget)->active)?TRUE:FALSE,
		NULL);
}

void Monitor::sidepanel()
{
	if (GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[0].widget)->active)
	{
		gtk_widget_show(hpaned1);
		gtk_widget_ref(notebook2);
		if (notebook2->parent == vbox1)
			gtk_container_remove(GTK_CONTAINER(vbox1), notebook2);
		if (notebook2->parent == NULL)
			gtk_container_add(GTK_CONTAINER(hpaned1), notebook2);
		gtk_widget_unref(notebook2);
	}else
	{
		gtk_widget_hide(hpaned1);
		gtk_widget_ref(notebook2);
		if (notebook2->parent == hpaned1)
			gtk_container_remove(GTK_CONTAINER(hpaned1), notebook2);
		if (notebook2->parent == NULL)
			gtk_container_add(GTK_CONTAINER(vbox1), notebook2);
		gtk_widget_unref(notebook2);
	}
}

void Monitor::statusbar_active()
{
	gconf_client_set_bool(main->client, CONFIG_SHOW_STATUSBAR,
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[1].widget)->active)?TRUE:FALSE,
		NULL);
}

void Monitor::statbar()
{
	if (GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[1].widget)->active)
	{
		gtk_widget_show(statusbar);
	}else
	{
		gtk_widget_hide(statusbar);
	}
}

void Monitor::labels_active()
{
	gconf_client_set_bool(main->client, CONFIG_DIAGRAM_SHOW_LABELS,
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[2].widget)->active)?TRUE:FALSE,
		NULL);
}

void Monitor::labels()
{
	viewer->diagram->showlabels =
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[2].widget)->active)?TRUE:FALSE;
	viewer->draw();
}

void Monitor::colour_active()
{
	gconf_client_set_bool(main->client, CONFIG_DIAGRAM_SHOW_LINE_COLORS,
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[3].widget)->active)?TRUE:FALSE,
		NULL);
}

void Monitor::colour()
{
	viewer->diagram->showcolourlines =
		(GTK_CHECK_MENU_ITEM(settings1_menu_uiinfo[3].widget)->active)?TRUE:FALSE;
	viewer->draw();
}

void Monitor::create_statusbar(GtkWidget *vbox)
{
	GtkWidget *frame;
	GtkWidget *hbox;

	statusbar = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (statusbar);
  gtk_box_pack_end (GTK_BOX (vbox), statusbar, FALSE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (statusbar), 1);

  frame = gtk_frame_new (NULL);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, TRUE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);

	message = gtk_label_new("");
	gtk_widget_show (message);
  gtk_container_add (GTK_CONTAINER (frame), message);
  gtk_misc_set_alignment (GTK_MISC (message), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (message), 2, 0);

  counter_frame = gtk_frame_new (NULL);
  gtk_widget_show (counter_frame);
  gtk_box_pack_start (GTK_BOX (statusbar), counter_frame, FALSE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (counter_frame), GTK_SHADOW_IN);

	hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER(counter_frame), hbox);

	counter[0] = gtk_label_new ("0");
	counter[1] = gtk_label_new ("0");
	counter[2] = gtk_label_new ("0");
	counter[3] = gtk_label_new ("0");

	statusbar_variable(hbox, counter[0], "Devices", pixmap[8]);
	statusbar_variable(hbox, counter[1], "Hosts", pixmap[5]);
	statusbar_variable(hbox, counter[2], "Network Clouds", pixmap[6]);
	statusbar_variable(hbox, counter[3], "Subnets", pixmap[7]);

	frame = gtk_frame_new (NULL);
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, FALSE, TRUE, 0);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);

  task_button = gtk_button_new_with_label ("0 Tasks");
  gtk_widget_show (task_button);
  gtk_container_add (GTK_CONTAINER (frame), task_button);
  gtk_tooltips_set_tip (tooltips, task_button, "Task List", NULL);
  gtk_button_set_relief (GTK_BUTTON (task_button), GTK_RELIEF_NONE);
	gtk_signal_connect (GTK_OBJECT (task_button), "pressed",
		GTK_SIGNAL_FUNC (task_cb), this);
}

void Monitor::statusbar_variable(GtkWidget *box, GtkWidget *label, char *tip, GtkWidget *pix)
{
	GtkWidget *eventbox;
	GtkWidget *hbox;

	eventbox = gtk_event_box_new ();
  gtk_widget_show (eventbox);
  gtk_box_pack_start (GTK_BOX (box), eventbox, FALSE, TRUE, 2);
  gtk_tooltips_set_tip (tooltips, eventbox, tip, NULL);

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox);
  gtk_container_add (GTK_CONTAINER (eventbox), hbox);

  gtk_box_pack_start (GTK_BOX (hbox), pix, FALSE, TRUE, 0);

  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
}

void Monitor::statusbar_update()
{
	char buf[64];

	if (main->session->data!=NULL)
	{
		sprintf(buf, "%d", main->session->data->devices.size());
		gtk_label_set_text(GTK_LABEL(counter[0]), buf);
		sprintf(buf, "%d", main->session->data->machines.size());
		gtk_label_set_text(GTK_LABEL(counter[1]), buf);
		sprintf(buf, "%d", main->session->data->networks.size());
		gtk_label_set_text(GTK_LABEL(counter[2]), buf);
		sprintf(buf, "%d", main->session->data->subnets.size());
		gtk_label_set_text(GTK_LABEL(counter[3]), buf);
	}
	gtk_widget_set_usize(counter_frame, 0, 0);
}

void Monitor::show_message(char *buf)
{
	// printf("HUMAN MESSAGE: %s\n", buf);
	gtk_label_set_text(GTK_LABEL(message), buf);
}

void Monitor::show_task_label(int size)
{
	char buf[16];

	sprintf(buf, "%d task%s", size, (size!=1)?"s":"");
	gtk_label_set_text(GTK_LABEL(GTK_BIN(task_button)->child), buf);
}

/*
void Preferences::load_settings()
{
	gint x, y, width, height;

	gdk_window_get_root_origin (main->gui->monitor->mainwindow, &x, &y);
	gdk_window_get_size (main->gui->monitor->mainwindow, &width, &height);

	state_push_prefix (name);
	gnome_config_set_int ("_x=", x);
	gnome_config_set_int ("_y=", y);
	gnome_config_set_int ("_width=", width);
	gnome_config_set_int ("_height=", height);
	gnome_config_pop_prefix ();
	gnome_config_sync ();
}

void
state_load_window_state (GtkWidget *window, const gchar *name,
			int defwidth, int defheight)
{
	GString *buf;
	gint x, y, width, height;
	
	buf = g_string_sized_new (16);
	
	state_push_prefix (name);

	x = gnome_config_get_int ("_x=-1");
	y = gnome_config_get_int ("_y=-1");
	if (!(x == -1 && y == -1))
		gtk_widget_set_uposition (window, x, y);

	g_string_sprintf (buf, "_width=%i", defwidth);
	width = gnome_config_get_int (buf->str);
	g_string_sprintf (buf, "_height=%i", defheight);
	height = gnome_config_get_int (buf->str);
	if (!(width == -1 && height == -1))
		gtk_window_set_default_size (GTK_WINDOW (window),
					     width, height);

	g_string_free (buf, TRUE);
	gnome_config_pop_prefix ();
}

*/

void Monitor::import_action()
{
	gchar *fname;

	fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel->window));
	filesel->hide();
	main->session->import_init(fname);
	main->session->import(TRUE);
}

void Monitor::import_list_action()
{
	FILE *f;
	gchar *fname, *buf2, *csl;
	gchar buf[256];
	struct hostent *h;
	IPAddress tip;

	fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel2->window));
	filesel2->hide();

	// printf("FILE %s has %d lines\n", fname, linesInFile(fname));

	f = fopen(fname, "r");
	if (f)
	{
		csl = get_community_list_string(main->client);
		while(fscanf(f, "%[^\n]\n", buf)==1)
		{
			if ((h=gethostbyname(buf)) != NULL)
			{
				tip.setstring(inet_ntoa(*((struct in_addr *)h->h_addr)));
				if (!tip.zero() && !tip.loopback())
				{
					buf2 = g_strjoin(" ", tip.getstring(), csl, NULL);
					main->scheduler->addTask(buf2, 1, 1, 0);
					g_free(buf2);
				}
			}
		}
		fclose(f);
		g_free(csl);
	}
}

void Monitor::refresh()
{
	if (viewer->refresh)
	{
		viewer->draw();
		viewer->refresh = FALSE;
	}
}

