/* preferences.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef PREFS_H
#define PREFS_H 1

#include <gconf/gconf-client.h>
#include <gtk/gtk.h>
#include "list.h"
#include "header.h"

class SampleRate
{
	public:
		guint32 frequency;
		guint32 duration;
};

class Main;
class Rule;
class Level;
class TimeFrame;
class Preferences
{
	public:
		Preferences(Main *m);
		~Preferences();

		void show();
		void show_community();
		void hide();
		void empty();

		void set_gconf_from_spinbutton(GtkWidget *w, gchar *config);
		void set_spinbutton_from_gconf(GtkWidget *w, gchar *config);
		void set_gconf_from_editable(GtkWidget *w, gchar *config);
		void set_editable_from_gconf(GtkWidget *w, gchar *config);
		void set_gconf_from_togglebutton(GtkWidget *w, gchar *config);
		void set_togglebutton_from_gconf(GtkWidget *w, gchar *config);
		void set_gconf_from_color(GtkWidget *w, gchar *config);
		void set_color_from_gconf(GtkWidget *w, gchar *config);
		void set_gconf_from_font(GtkWidget *w, gchar *config);
		void set_font_from_gconf(GtkWidget *w, gchar *config);
		void set_time_from_gconf(GtkWidget *w, GtkWidget *w2, gchar *key);
		void set_gconf_from_time(GtkWidget *w, GtkWidget *w2, gchar *key);
		void set_simple_clist_from_gconf(GtkWidget *clist, gchar *key);
		void set_gconf_from_simple_clist(GtkWidget *clist, gchar *key);

		void config_update(gchar *key, gint refresh);
		void config_widget_action(GtkWidget *w);
		void config_clist_select_action(GtkWidget *w, int row);
		void config_clist_unselect_action(GtkWidget *w, int row);

		void global_init();

		Main *main;
		GtkWidget *window;
		GtkWidget *sidebar;
		GtkWidget *notebook;
		GSList *button_group;
		GtkWidget *current_panel;
		GtkWidget *monitor_button;
		GtkWidget *community_button;
		GtkWidget *close_button;
		void build_sidebar();
		GtkWidget *sidebar_button(char *label);
		void sidebar_icon(GtkWidget *w, char *label, char *fname, int pg);
		void sidebar_pad(GtkWidget *w);
		void sidebar_button_pressed(GtkWidget *w);
		void sidebar_icon_pressed(GtkWidget *w);

		GtkWidget *session_name_entry;
		GtkWidget *session_auto_save;
		GtkWidget *session_auto_save_time1;
		GtkWidget *session_auto_save_time2;
		GtkWidget *session_save_on_exit;
//		GtkWidget *session_store_topology;
		GtkWidget *session_ping_time1;
		GtkWidget *session_ping_time2;
		GtkWidget *session_regrab_time1;
		GtkWidget *session_regrab_time2;
		GtkWidget *session_topology_default;
		GtkWidget *session_graph_default;

		GtkWidget *import_clist;
		GtkWidget *import_filename;
		GtkWidget *import_auto;
		GtkWidget *import_time1;
		GtkWidget *import_time2;
		GtkWidget *import_add;
		GtkWidget *import_delete;
		GtkWidget *import_import;
		int import_row;
		void import_add_action();
		void import_del_action();
		void import_import_action(gboolean show_progress);

		GtkWidget *support_ping_entry;
		GtkWidget *support_telnet_entry;
		GtkWidget *support_traceroute_entry;

		GtkWidget *search_zoom_checkbutton;
		GtkWidget *search_hide_checkbutton;
		GtkWidget *search_hidden_checkbutton;
		GtkWidget *search_all_checkbutton;

		GtkWidget *default_animate_zoom;
 		GtkWidget *default_show_leaf;
 		GtkWidget *default_show_empty;
		GtkWidget *default_group_leaf_type;
		GtkWidget *default_network_type;
		GtkWidget *default_network_position;
		GtkWidget *default_show_machines;
		GtkWidget *default_group_machines;

		GtkWidget *font_default;
		GtkWidget *color_background;
		GtkWidget *color_list1;
		GtkWidget *color_list2;
		GtkWidget *color_device_up;
		GtkWidget *color_device_down;
		GtkWidget *color_selected_device_up;
		GtkWidget *color_selected_device_down;
		GtkWidget *color_machine;
		GtkWidget *color_selected_machine;
		GtkWidget *color_network;
		GtkWidget *color_empty_network;
		GtkWidget *color_selected_network;

		GtkWidget *community_clist;
		GtkWidget *community_entry;
		GtkWidget *community_add_button;
		GtkWidget *community_delete_button;
		int community_row;
		void community_add_action();
		void community_del_action();

		GtkWidget *sample_clist;
		GtkWidget *sample_label;
		GtkWidget *sample_error_label;
		GtkWidget *sample_frequency1;
		GtkWidget *sample_frequency2;
		GtkWidget *sample_duration1;
		GtkWidget *sample_duration2;
		GtkWidget *sample_add;
		GtkWidget *sample_delete;
		GtkWidget *sample_errors1;
		GtkWidget *sample_errors2;
		GtkWidget *sample_rate1;
		GtkWidget *sample_rate2;
		GtkWidget *sample_replace;
		GtkWidget *sample_apply;
		GtkWidget *sample_revert;
		int sample_row;
		List samples;
		void sample_load();
		void sample_save();
		void sample_refresh();
		void sample_refresh2();
		void sample_add_action();
		void sample_delete_action();
		void sample_apply_action();
		void sample_revert_action();

		GtkWidget *timeframe_new_button;
		GtkWidget *timeframe_delete_button;
		GtkWidget *timeframe_name_label;
		GtkWidget *timeframe_mon_checkbutton;
		GtkWidget *timeframe_tue_checkbutton;
		GtkWidget *timeframe_wed_checkbutton;
		GtkWidget *timeframe_thu_checkbutton;
		GtkWidget *timeframe_fri_checkbutton;
		GtkWidget *timeframe_sat_checkbutton;
		GtkWidget *timeframe_sun_checkbutton;
		GtkWidget *timeframe_invert_checkbutton;
		GtkWidget *timeframe_name_textentry;
		GtkWidget *timeframe_start_combo;
		GtkWidget *timeframe_end_combo;
		GtkWidget *timeframe_clist;
		GtkWidget *timeframe_table;
		List timeframes;
		int current_timeframe;
		gint timeframe_add(TimeFrame *t);
		void timeframe_new_button_action();
		void timeframe_delete_button_action();
		void timeframe_clist_select_action(gint row);
		void timeframe_clist_unselect_action();
		void timeframe_name_textentry_action();
		void timeframe_invert_checkbutton_action();
		void timeframe_start_combo_action();
		void timeframe_end_combo_action();
		void timeframe_day_action(GtkWidget *w);
		void timeframe_day_string(int row);

		GtkWidget *level_clist;
		GtkWidget *level_name_label;
		GtkWidget *level_name_entry;
		GtkWidget *level_color_label;
		GtkWidget *level_color_picker;
		GtkWidget *level_new_button;
		GtkWidget *level_delete_button;
		List levels;
		int current_level;
		gint level_add(Level *l);
		void level_new_button_action();
		void level_delete_button_action();
		void level_clist_select_action(gint row);
		void level_clist_unselect_action();
		void level_name_entry_action();
		void level_color_picker_action();

		GtkWidget *action_frame;
		GtkWidget *action_clist;
		GtkWidget *action_type_combo;
		GtkWidget *action_new_button;
		GtkWidget *action_delete_button;
		GtkWidget *action_test_button;
		GtkWidget *action_notebook;
		GtkWidget *action_command_line_entry;
		GtkWidget *action_email_address_entry;
		GtkWidget *action_sound_file_entry;
		int current_action;
		void action_init(Level *l);
		void action_new_button_action();
		void action_delete_button_action();
		void action_test_button_action();
		void action_type_combo_action();
		void action_clist_select_action(int row);
		void action_clist_unselect_action();
		void action_command_line_entry_action();
		void action_email_address_entry_action();
		void action_sound_file_entry_action();

		GtkWidget *notify_clist;
		GtkWidget *notify_new_button;
		GtkWidget *notify_delete_button;
		GtkWidget *notify_event_type_combo;
		GtkWidget *notify_threshold_entry;
		GtkWidget *notify_percentage_checkbutton;
		GtkWidget *notify_offset_checkbutton;
		GtkWidget *notify_occurance_spinbutton;
		GtkWidget *notify_time_frame_combo;
		GtkWidget *notify_level_combo;
		GtkWidget *notify_table;
		GtkWidget *notify_hbox69;
		GtkWidget *notify_hbox75;
		List rules;
		int current_rule;
		void notify_event_type_list();
		gint rules_add(Rule *r);
		void notify_new_button_action();
		void notify_delete_button_action();
		void notify_clist_select_action(int row);
		void notify_clist_unselect_action();
		void notify_event_type_combo_action();
		void notify_level_combo_action();
		void notify_time_frame_combo_action();
		void notify_threshold_entry_action();
		void notify_percentage_checkbutton_action();
		void notify_offset_checkbutton_action();
		void notify_occurance_spinbutton_action();

		GtkWidget *email_sender_address;
		GtkWidget *email_smtp_server;
		GtkWidget *email_port_number;
		GtkWidget *email_message_subject;
		GtkWidget *email_message_body;

		void save_xml();
		void load_xml();

		gboolean import_auto_flag;
		gboolean autosave_flag;
		gboolean save_on_quit_flag;
};

gchar *get_community_list_string(GConfClient *client);

#endif

