/* scheduler.h
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef SCHEDULER_H
#define SCHEDULER_H 1

#include "timestamp.h"
#include "list.h"
#include "device.h"

#define BUFSIZE 256

class Main;
class Process;
class Scheduler
{
	public:
		Scheduler(Main *m);
		~Scheduler();

		void timeout();
		void addTask(char *str, int type, int lvl, int hop);
		void delTasks(int type);
		void returnedTask(Process *p);
		void assignTask(Process *p);
		void keepaliveTask(Process *p);
		void pingcode();
		void human_message(char *str, int type);
		void regrabAllDevices();
		void regrabSelectedDevices();
		void regrabDevice(Device *d);

		Main *main;
		List tasks;
		List eventlevels;
		List timeframes;
		List rules;

		// Processes
		Process *alive;			// used to run alive script
		Process *discover1;	// used to perform discover and update tasks
		Process *discover2;	// used to perform discover and update tasks
		Process *discover3;	// used to perform discover and update tasks

		// Timers
		int ping_interval;
		int topology_interval;
		int autosave_interval;
		int import_interval;
		int stats_interval;
		TimeStamp ping_timer;
		TimeStamp topology_timer;
		TimeStamp autosave_timer;
		TimeStamp import_timer;
		TimeStamp stats_timer;
};

#endif

