/*****************************************************************************
 Copyright (C) Paul Coates
 *****************************************************************************/

#include <stdio.h>
#include "list.h"

List::List()
{
	count = 0;
	first = NULL;
	last = NULL;
}

List::~List()
{
	empty();
}

void List::add(void *i)
{
	Link *lnk = new Link();
	lnk->item = i;
	lnk->next = NULL;
	lnk->prev = NULL;

	if (first == NULL)
	{
		first = last = lnk;
	}else
	{
		first->prev = lnk;
		lnk->next = first;
		first = lnk;
	}
	count++;
}

void List::tag(void *i)
{
	Link *lnk = new Link();
	lnk->item = i;
	lnk->next = NULL;
	lnk->prev = NULL;

	if (last == NULL)
	{
		first = last = lnk;
	}else
	{
		last->next = lnk;
		lnk->prev = last;
		last = lnk;
	}
	count++;
}

// Delete the link that points to the given object

void List::del(void *i)
{
	Link *lk, *lnk = first;

	while (lnk!=NULL)
	{
		lk = lnk->next;
		if (i == lnk->item)
		{
			if (first == lnk) first = lnk->next;
			if (last == lnk) last = lnk->prev;
			if (lnk->prev != NULL) lnk->prev->next = lnk->next;
			if (lnk->next != NULL) lnk->next->prev = lnk->prev;
			delete lnk;
			count--;
		}
		lnk = lk;
	}
}

// Delete things from the list without worrying about the links
// just run tidy to remove those links that have NULL pointers

void List::tidy()
{
	Link *lk, *lnk = first;

	while (lnk!=NULL)
	{
		lk = lnk->next;
		if (lnk->item==NULL)
		{
			if (first == lnk) first = lnk->next;
			if (last == lnk) last = lnk->prev;
			if (lnk->prev != NULL) lnk->prev->next = lnk->next;
			if (lnk->next != NULL) lnk->next->prev = lnk->prev;
			delete lnk;
			count--;
		}
		lnk = lk;
	}
}

// Remove all links from the list

void List::empty()
{
	Link *lnk = first;

	while (lnk!=NULL)
	{
		lnk = first->next;
		delete first;
		first = lnk;
	}
	last = NULL;
	count = 0;
}

// Remove all links from the list AND all those items they pointed to

/*
void List::wipe()
{
	Link *lnk = first;

	while (lnk!=NULL)
	{
		if (lnk->item!=NULL)
			delete lnk->item;
		lnk = first->next;
		delete first;
		first = lnk;
	}
	last = NULL;
	count = 0;
}
*/

void List::raise(void *i)
{
	del(i);
	add(i);
}

void List::lower(void *i)
{
	del(i);
	tag(i);
}

void List::up(void *i)
{
	Link *lk, *lk2, *lnk = first;

	while (lnk!=NULL)
	{
		lk = lnk->next;
		if (i == lnk->item)
		{
			// found item to move
			if (first == lnk) return;
			lk2 = lnk->prev;
			if (lk2->prev != NULL) lk2->prev->next = lnk;
			if (lnk->next != NULL) lnk->next->prev = lk2;
			lnk->prev = lk2->prev;
			lk2->next = lnk->next;
			lnk->next = lk2;
			lk2->prev = lnk;
			if (lk2 == first) first = lnk;
			if (lnk == last) last = lk2;
			return;
		}
		lnk = lk;
	}
}

void List::down(void *i)
{
	Link *lk, *lk2, *lnk = first;

	while (lnk!=NULL)
	{
		lk = lnk->next;
		if (i == lnk->item)
		{
			// found item to move
			if (last == lnk) return;
			lk2 = lnk->next;
			if (lk2->next != NULL) lk2->next->prev = lnk;
			if (lnk->prev != NULL) lnk->prev->next = lk2;
			lnk->next = lk2->next;
			lk2->prev = lnk->prev;
			lnk->prev = lk2;
			lk2->next = lnk;
			if (lk2 == last) last = lnk;
			if (lnk == first) first = lk2;
			return;
		}
		lnk = lk;
	}
}

void *List::get_item(int i)
{
	Link *current = first;
	while(i>0)
	{
		if (current) current = current->next;
		i--;
	}
	if (current) return current->item;
	return NULL;
}

Link *List::get_head()
{
	return first;
}

Link *List::get_tail()
{
	return last;
}

int List::size()
{
	return count;
}

void swap(Link *a, Link *b)
{
	void *c = a->item;
	a->item = b->item;
	b->item = c;	
}

void List::sort(Compare compare, int type)
{
	switch (type)
	{
		case BUBBLE_SORT: bubblesort(compare, first, last); break;
		default: break;
	}
}

void List::bubblesort(Compare compare, Link *low, Link *high)
{
	Link *ln, *ln2;

	if (high==NULL) return;
	if (low==NULL) return;
	if (low==high) return;

	for (ln=low; ln!=NULL; ln=ln->next)
	{
		for (ln2=ln->next; ln2!=NULL; ln2=ln2->next)
		{
				if (compare(ln, ln2)>0) swap(ln, ln2);
		}
	}
}

