#include <string.h>
#include <stdlib.h>
#include "header.h"
#include "network.h"
#include "device.h"
#include "subnet.h"
#include "interface.h"
#include "machine.h"
#include "filter.h"
#include "misc.h"

Network::Network()
{
	empty();
}

Network::~Network()
{
	empty();
}

void Network::empty()
{
	Interface *in;
	Machine *m;
	MACAddress *mac;
	Subnet *s;
	Link *ln;

	x = rand()%4000;
	y = rand()%3000;
	w = h = 96;
	selection = FALSE;
	shown = TRUE;
	maintype = visual_network;
	subtype = network_cloud;
	group = network_group_bunch;
	position = network_position_auto;
	interfaces.empty();
}

void Network::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	Interface *in;
	Device *d;
	Link *ln;
	char buf[256];

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Network", NULL);
	strcpy(buf, networkTypeToString(subtype));
	xmlSetProp(newxml, (xmlChar*)"group", (xmlChar*)buf);
	strcpy(buf, networkPositionTypeToString(position));
	xmlSetProp(newxml, (xmlChar*)"position", (xmlChar*)buf);
	strcpy(buf, networkGroupTypeToString(group));
	xmlSetProp(newxml, (xmlChar*)"group", (xmlChar*)buf);
	sprintf(buf, "%d", x);
	xmlSetProp(newxml, (xmlChar*)"x", (xmlChar*)buf);
	sprintf(buf, "%d", y);
	xmlSetProp(newxml, (xmlChar*)"y", (xmlChar*)buf);
	sprintf(buf, "%s", (shown==TRUE)?"yes":"no");
	xmlSetProp(newxml, (xmlChar*)"show", (xmlChar*)buf);

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Iface", NULL);
		in = (Interface*) ln->item;
		if (in!=NULL)
		{
			d = (Device*) in->device;
			if (d!=NULL)
			{
				xmlSetProp(cur, (xmlChar*)"router", (xmlChar*)d->pingname.getstring());
				sprintf(buf, "%d", in->index);
				xmlSetProp(cur, (xmlChar*)"index", (xmlChar*)buf);
			}
		}
	}
}

void Network::loadxml(xmlNodePtr xmlnode, List *rtr, List *ntwk)
{
	xmlChar *buf;
	Link *ln, *ln2;
	Subnet sn, *sn2;
	Machine *m;
	IPAddress ip;
	Interface *in;
	MACAddress *mac;
	Device *d;
	char tname[65];
	int index;

	// check this is a router node
	if (!xmlnode || !xmlnode->name || strcmp((char*)xmlnode->name, "Network")!=0)
		return;
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"x");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &x); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"y");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &y); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"show");
	if (buf!=NULL)
	{ shown = (strcmp((gchar*)buf, "yes")==0)?TRUE:FALSE; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"group");
	if (buf!=NULL) { subtype = stringToNetworkType((gchar*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"position");
	if (buf!=NULL)
	{ position = stringToNetworkPositionType((gchar*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"group");
	if (buf!=NULL) { group = stringToNetworkGroupType((gchar*)buf); free(buf); }

	interfaces.empty();

	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Iface")==0)
			{
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"router");
				if (buf!=NULL) { strcpy(tname, (char*)buf); free(buf); }
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"index");
				if (buf!=NULL) { sscanf((char*)buf, "%d", &index); free(buf); }
				ip.setstring(tname);

				for (ln=rtr->get_head(); ln!=NULL; ln=ln->next)
				{
					d = (Device*) ln->item;
					if (d!=NULL)
					{
						if (ip.compare(d->pingname)==0)
						{
							for (ln2=d->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
							{
								in = (Interface*) ln2->item;
								if (in!=NULL)
								{
									if (in->index == index)
									{
										if (in->network!=NULL)
										{
											ntwk->del(in->network);
											delete in->network;
										}
										in->network = this;
										interfaces.tag(in);
									}
								}
							}
						}
					}
				}
			}
		}
		xmlnode = xmlnode->next;
	}
}

void Network::matchfilter(Filter *f)
{
/*
	Link *ln, *ln2;
	IPAddress *tip;
	Interface *in;
	Machine *m;

	if (f->default_action==action_deny)
		show='0';
	else
		show='1';

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		for (ln2=in->ip_addresses.get_head(); ln2!=NULL; ln2=ln2->next)
		{
			tip = (IPAddress*) ln2->item;
			if (f->default_action==action_deny)
			{
				if (f->match(*tip)!=0) { show='1'; return; }
			}else
			{
				if (f->match(*tip)==0) { show='0'; return; }				
			}
		}
	}
	for (ln=machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if (f->default_action==action_deny)
		{
			if (f->match(m->ip)!=0) { show='1'; return; }
		}else
		{
			if (f->match(m->ip)==0) { show='0'; return; }				
		}
	}
	// could also check subnets but should be already covered
*/
}

gint Network::numMachines()
{
	gint count;
	Interface *in;
	Link *ln;

	count = 0;
	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		count += in->machines.size();
	}
	return count;
}

gboolean Network::matchFDDI()
{
	Link *ln;
	Interface *in;

	for (ln=interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		if (in->type==15) return TRUE;
	}

	return FALSE;
}

